require "avro"
require "kafka"
require "avro_turf/messaging"
require "virtus"
require "qian/util"
require "qian/event"
require "qian/logger"
require "active_support"
require "active_support/core_ext"
require "qian/events/engagement"

module Qian
  extend self

  attr_accessor :logger_config
  attr_accessor :kafka_config
  attr_accessor :schema_registry_url
  attr_accessor :schema_path
  attr_accessor :disabled

  #
  # 当前 Qian lib 的路径
  #
  #
  # @return [Pathname]
  #
  def root
    Pathname.new(File.expand_path(File.dirname(__FILE__)))
  end

  #
  # 设置 Qian 配置
  #
  #
  # @return []
  #
  def configure(&blk)
    blk.yield(Qian)

    #
    # Setting up
    #
    @kafka ||= Kafka.new(kafka_config[:connection])
    @avro ||= AvroTurf::Messaging.new(
      :registry_url => schema_registry_url, :schemas_path => schema_path)
  end

  #
  # 是否禁用
  #
  #
  # @return [Boolean]
  #
  def disabled?
    self.disabled
  end

  #
  # 返回一个 Kafka Async Producer 实例
  #
  #
  # @return [Kafka::AsyncProducer]
  #
  def kafka_producer
    return nil if disabled?
    @producer ||= @kafka.async_producer(kafka_config[:async_config])
  end

  #
  # 返回 Qian::Logger
  #
  #
  # @return [Qian::Logger]
  #
  def logger
    @logger || Qian::Logger.new(kafka_producer)
  end

  #
  # 返回 Avro Instance
  #
  #
  # @return [Avro]
  #
  def avro
    @avro
  end

  #
  # 资源清理
  #
  #
  # @return [void]
  #
  def shutdown
    # 关闭 Kafka Producer
    @producer.present? && @producer.shutdown
  end

  #
  # Default Settings
  #
  self.kafka_config = {
    :connection => {
      :seed_brokers => ["localhost:9092"],
      :client_id => "kafka-rb-producer"
    },
    :async => {
      :delivery_threshold => 2_000,
      :delivery_interval => 30,
      :max_buffer_size => 2_000,
      :max_buffer_bytesize => 100_000_000
    }
  }

  self.logger_config = {
    :topic => "admin-app-log"
  }

  self.schema_registry_url = "http://127.0.0.1:28081/"
  self.schema_path = Qian.root.join("..", "schema")
  self.disabled = false
end
